#!/usr/bin/env perl

# [[[ PREPROCESSOR ]]]
# <<< EXECUTE_SUCCESS: 'Pragma "use Perl::Types;"' >>>

# [[[ HEADER ]]]
use strict;
use warnings;
use Perl::Types;
our $VERSION = 0.001_000;

# [[[ OPERATIONS ]]]
print 'Pragma "use Perl::Types;"', "\n";
